from __future__ import annotations

from typing import Optional, TYPE_CHECKING

from sandbox0.apispec.api.sandboxes import delete_api_v1_sandboxes_id
from sandbox0.apispec.api.sandboxes import get_api_v1_sandboxes_id
from sandbox0.apispec.api.sandboxes import get_api_v1_sandboxes_id_status
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes_id_pause
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes_id_refresh
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes_id_resume
from sandbox0.apispec.api.sandboxes import put_api_v1_sandboxes_id
from sandbox0.apispec.api.sandbox_volumes import delete_api_v1_sandboxvolumes_id
from sandbox0.apispec.api.sandbox_volumes import get_api_v1_sandboxvolumes
from sandbox0.apispec.api.sandbox_volumes import get_api_v1_sandboxvolumes_id
from sandbox0.apispec.api.sandbox_volumes import post_api_v1_sandboxvolumes
from sandbox0.apispec.api.snapshots import delete_api_v1_sandboxvolumes_id_snapshots_snapshot_id
from sandbox0.apispec.api.snapshots import get_api_v1_sandboxvolumes_id_snapshots
from sandbox0.apispec.api.snapshots import get_api_v1_sandboxvolumes_id_snapshots_snapshot_id
from sandbox0.apispec.api.snapshots import post_api_v1_sandboxvolumes_id_snapshots
from sandbox0.apispec.api.snapshots import post_api_v1_sandboxvolumes_id_snapshots_snapshot_id_restore
from sandbox0.apispec.models.claim_request import ClaimRequest
from sandbox0.apispec.models.create_sandbox_volume_request import CreateSandboxVolumeRequest
from sandbox0.apispec.models.create_snapshot_request import CreateSnapshotRequest
from sandbox0.apispec.models.pause_sandbox_response import PauseSandboxResponse
from sandbox0.apispec.models.refresh_request import RefreshRequest
from sandbox0.apispec.models.refresh_response import RefreshResponse
from sandbox0.apispec.models.resume_sandbox_response import ResumeSandboxResponse
from sandbox0.apispec.models.sandbox import Sandbox as APISandbox
from sandbox0.apispec.models.sandbox_config import SandboxConfig
from sandbox0.apispec.models.sandbox_status import SandboxStatus
from sandbox0.apispec.models.sandbox_update_request import SandboxUpdateRequest
from sandbox0.apispec.models.sandbox_volume import SandboxVolume
from sandbox0.apispec.models.snapshot import Snapshot
from sandbox0.apispec.models.success_claim_response import SuccessClaimResponse
from sandbox0.apispec.models.success_deleted_response import SuccessDeletedResponse
from sandbox0.apispec.models.success_message_response import SuccessMessageResponse
from sandbox0.apispec.models.success_pause_sandbox_response import SuccessPauseSandboxResponse
from sandbox0.apispec.models.success_refresh_response import SuccessRefreshResponse
from sandbox0.apispec.models.success_resume_sandbox_response import SuccessResumeSandboxResponse
from sandbox0.apispec.models.success_restore_response import SuccessRestoreResponse
from sandbox0.apispec.models.success_sandbox_response import SuccessSandboxResponse
from sandbox0.apispec.models.success_sandbox_status_response import SuccessSandboxStatusResponse
from sandbox0.apispec.models.success_sandbox_volume_list_response import SuccessSandboxVolumeListResponse
from sandbox0.apispec.models.success_sandbox_volume_response import SuccessSandboxVolumeResponse
from sandbox0.apispec.models.success_snapshot_list_response import SuccessSnapshotListResponse
from sandbox0.apispec.models.success_snapshot_response import SuccessSnapshotResponse
from sandbox0.response import ensure_data, ensure_model
from sandbox0.sessions import SandboxSession, VolumeSession

if TYPE_CHECKING:
    from sandbox0.client import Client
    from sandbox0.sandbox import Sandbox


class Sandboxes:
    def __init__(self, client: "Client") -> None:
        self._client = client

    def claim(self, template: str, config: Optional[SandboxConfig] = None) -> "Sandbox":
        request = ClaimRequest(template=template, config=config) if config is not None else ClaimRequest(template=template)
        resp = post_api_v1_sandboxes.sync_detailed(client=self._client.api, body=request)
        data = ensure_data(resp, SuccessClaimResponse)
        from sandbox0.sandbox import Sandbox

        return Sandbox(
            id=data.sandbox_id,
            client=self._client,
            template=data.template,
            cluster_id=None if data.cluster_id.__class__.__name__ == "Unset" else data.cluster_id,
            pod_name=data.pod_name,
            status=data.status,
        )

    def open(self, template: str, config: Optional[SandboxConfig] = None) -> SandboxSession:
        sandbox = self.claim(template, config=config)
        return SandboxSession(sandbox, closer=lambda: self.delete(sandbox.id))

    def get(self, sandbox_id: str) -> APISandbox:
        resp = get_api_v1_sandboxes_id.sync_detailed(id=sandbox_id, client=self._client.api)
        return ensure_data(resp, SuccessSandboxResponse)

    def update(self, sandbox_id: str, request: SandboxUpdateRequest) -> APISandbox:
        resp = put_api_v1_sandboxes_id.sync_detailed(id=sandbox_id, client=self._client.api, body=request)
        return ensure_data(resp, SuccessSandboxResponse)

    def delete(self, sandbox_id: str) -> SuccessMessageResponse:
        resp = delete_api_v1_sandboxes_id.sync_detailed(id=sandbox_id, client=self._client.api)
        return ensure_model(resp, SuccessMessageResponse)

    def status(self, sandbox_id: str) -> SandboxStatus:
        resp = get_api_v1_sandboxes_id_status.sync_detailed(id=sandbox_id, client=self._client.api)
        return ensure_data(resp, SuccessSandboxStatusResponse)

    def pause(self, sandbox_id: str) -> PauseSandboxResponse:
        resp = post_api_v1_sandboxes_id_pause.sync_detailed(id=sandbox_id, client=self._client.api)
        return ensure_data(resp, SuccessPauseSandboxResponse)

    def resume(self, sandbox_id: str) -> ResumeSandboxResponse:
        resp = post_api_v1_sandboxes_id_resume.sync_detailed(id=sandbox_id, client=self._client.api)
        return ensure_data(resp, SuccessResumeSandboxResponse)

    def refresh(self, sandbox_id: str, request: Optional[RefreshRequest] = None) -> RefreshResponse:
        body = request if request is not None else RefreshRequest()
        resp = post_api_v1_sandboxes_id_refresh.sync_detailed(id=sandbox_id, client=self._client.api, body=body)
        return ensure_data(resp, SuccessRefreshResponse)

    def sandbox(self, sandbox_id: str) -> "Sandbox":
        from sandbox0.sandbox import Sandbox

        return Sandbox(id=sandbox_id, client=self._client)


class Volumes:
    def __init__(self, client: "Client") -> None:
        self._client = client

    def create(self, request: CreateSandboxVolumeRequest) -> SandboxVolume:
        resp = post_api_v1_sandboxvolumes.sync_detailed(client=self._client.api, body=request)
        return ensure_data(resp, SuccessSandboxVolumeResponse)

    def open(self, request: CreateSandboxVolumeRequest) -> VolumeSession:
        volume = self.create(request)
        return VolumeSession(volume, closer=lambda: self.delete(volume.id))

    def list(self) -> list[SandboxVolume]:
        resp = get_api_v1_sandboxvolumes.sync_detailed(client=self._client.api)
        data = ensure_data(resp, SuccessSandboxVolumeListResponse)
        return data

    def get(self, volume_id: str) -> SandboxVolume:
        resp = get_api_v1_sandboxvolumes_id.sync_detailed(id=volume_id, client=self._client.api)
        return ensure_data(resp, SuccessSandboxVolumeResponse)

    def delete(self, volume_id: str) -> SuccessDeletedResponse:
        resp = delete_api_v1_sandboxvolumes_id.sync_detailed(id=volume_id, client=self._client.api)
        return ensure_model(resp, SuccessDeletedResponse)

    def create_snapshot(self, volume_id: str, request: CreateSnapshotRequest) -> Snapshot:
        resp = post_api_v1_sandboxvolumes_id_snapshots.sync_detailed(id=volume_id, client=self._client.api, body=request)
        return ensure_data(resp, SuccessSnapshotResponse)

    def list_snapshots(self, volume_id: str) -> list[Snapshot]:
        resp = get_api_v1_sandboxvolumes_id_snapshots.sync_detailed(id=volume_id, client=self._client.api)
        data = ensure_data(resp, SuccessSnapshotListResponse)
        return data

    def get_snapshot(self, volume_id: str, snapshot_id: str) -> Snapshot:
        resp = get_api_v1_sandboxvolumes_id_snapshots_snapshot_id.sync_detailed(
            id=volume_id,
            snapshot_id=snapshot_id,
            client=self._client.api,
        )
        return ensure_data(resp, SuccessSnapshotResponse)

    def delete_snapshot(self, volume_id: str, snapshot_id: str) -> SuccessDeletedResponse:
        resp = delete_api_v1_sandboxvolumes_id_snapshots_snapshot_id.sync_detailed(
            id=volume_id,
            snapshot_id=snapshot_id,
            client=self._client.api,
        )
        return ensure_model(resp, SuccessDeletedResponse)

    def restore_snapshot(self, volume_id: str, snapshot_id: str) -> SuccessRestoreResponse:
        resp = post_api_v1_sandboxvolumes_id_snapshots_snapshot_id_restore.sync_detailed(
            id=volume_id,
            snapshot_id=snapshot_id,
            client=self._client.api,
        )
        return ensure_model(resp, SuccessRestoreResponse)



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateAPIKeyResponse")


@_attrs_define
class CreateAPIKeyResponse:
    """
    Attributes:
        id (str):
        name (str):
        type_ (str):
        roles (list[str]):
        team_id (str):
        expires_at (datetime.datetime):
        created_at (datetime.datetime):
        key (Union[Unset, str]):
    """

    id: str
    name: str
    type_: str
    roles: list[str]
    team_id: str
    expires_at: datetime.datetime
    created_at: datetime.datetime
    key: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        type_ = self.type_

        roles = self.roles

        team_id = self.team_id

        expires_at = self.expires_at.isoformat()

        created_at = self.created_at.isoformat()

        key = self.key

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
                "type": type_,
                "roles": roles,
                "team_id": team_id,
                "expires_at": expires_at,
                "created_at": created_at,
            }
        )
        if key is not UNSET:
            field_dict["key"] = key

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        type_ = d.pop("type")

        roles = cast(list[str], d.pop("roles"))

        team_id = d.pop("team_id")

        expires_at = isoparse(d.pop("expires_at"))

        created_at = isoparse(d.pop("created_at"))

        key = d.pop("key", UNSET)

        create_api_key_response = cls(
            id=id,
            name=name,
            type_=type_,
            roles=roles,
            team_id=team_id,
            expires_at=expires_at,
            created_at=created_at,
            key=key,
        )

        create_api_key_response.additional_properties = d
        return create_api_key_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.tpl_sandbox_network_policy_mode import TplSandboxNetworkPolicyMode
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.network_egress_policy import NetworkEgressPolicy


T = TypeVar("T", bound="TplSandboxNetworkPolicy")


@_attrs_define
class TplSandboxNetworkPolicy:
    """
    Attributes:
        mode (TplSandboxNetworkPolicyMode):
        egress (Union[Unset, NetworkEgressPolicy]):
    """

    mode: TplSandboxNetworkPolicyMode
    egress: Union[Unset, "NetworkEgressPolicy"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        mode = self.mode.value

        egress: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.egress, Unset):
            egress = self.egress.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "mode": mode,
            }
        )
        if egress is not UNSET:
            field_dict["egress"] = egress

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.network_egress_policy import NetworkEgressPolicy

        d = dict(src_dict)
        mode = TplSandboxNetworkPolicyMode(d.pop("mode"))

        _egress = d.pop("egress", UNSET)
        egress: Union[Unset, NetworkEgressPolicy]
        if isinstance(_egress, Unset):
            egress = UNSET
        else:
            egress = NetworkEgressPolicy.from_dict(_egress)

        tpl_sandbox_network_policy = cls(
            mode=mode,
            egress=egress,
        )

        tpl_sandbox_network_policy.additional_properties = d
        return tpl_sandbox_network_policy

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.file_content_response import FileContentResponse
    from ..models.file_info import FileInfo
    from ..models.success_file_read_response_data_type_1 import (
        SuccessFileReadResponseDataType1,
    )


T = TypeVar("T", bound="SuccessFileReadResponse")


@_attrs_define
class SuccessFileReadResponse:
    """
    Attributes:
        success (bool):
        data (Union['FileContentResponse', 'FileInfo', 'SuccessFileReadResponseDataType1', Unset]):
    """

    success: bool
    data: Union[
        "FileContentResponse", "FileInfo", "SuccessFileReadResponseDataType1", Unset
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.file_info import FileInfo
        from ..models.success_file_read_response_data_type_1 import (
            SuccessFileReadResponseDataType1,
        )

        success = self.success

        data: Union[Unset, dict[str, Any]]
        if isinstance(self.data, Unset):
            data = UNSET
        elif isinstance(self.data, FileInfo):
            data = self.data.to_dict()
        elif isinstance(self.data, SuccessFileReadResponseDataType1):
            data = self.data.to_dict()
        else:
            data = self.data.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "success": success,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_content_response import FileContentResponse
        from ..models.file_info import FileInfo
        from ..models.success_file_read_response_data_type_1 import (
            SuccessFileReadResponseDataType1,
        )

        d = dict(src_dict)
        success = d.pop("success")

        def _parse_data(
            data: object,
        ) -> Union[
            "FileContentResponse", "FileInfo", "SuccessFileReadResponseDataType1", Unset
        ]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                data_type_0 = FileInfo.from_dict(data)

                return data_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                data_type_1 = SuccessFileReadResponseDataType1.from_dict(data)

                return data_type_1
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            data_type_2 = FileContentResponse.from_dict(data)

            return data_type_2

        data = _parse_data(d.pop("data", UNSET))

        success_file_read_response = cls(
            success=success,
            data=data,
        )

        success_file_read_response.additional_properties = d
        return success_file_read_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.object_meta_annotations import ObjectMetaAnnotations
    from ..models.object_meta_labels import ObjectMetaLabels


T = TypeVar("T", bound="ObjectMeta")


@_attrs_define
class ObjectMeta:
    """
    Attributes:
        name (Union[Unset, str]):
        namespace (Union[Unset, str]):
        labels (Union[Unset, ObjectMetaLabels]):
        annotations (Union[Unset, ObjectMetaAnnotations]):
    """

    name: Union[Unset, str] = UNSET
    namespace: Union[Unset, str] = UNSET
    labels: Union[Unset, "ObjectMetaLabels"] = UNSET
    annotations: Union[Unset, "ObjectMetaAnnotations"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        namespace = self.namespace

        labels: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = self.labels.to_dict()

        annotations: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.annotations, Unset):
            annotations = self.annotations.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if namespace is not UNSET:
            field_dict["namespace"] = namespace
        if labels is not UNSET:
            field_dict["labels"] = labels
        if annotations is not UNSET:
            field_dict["annotations"] = annotations

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.object_meta_annotations import ObjectMetaAnnotations
        from ..models.object_meta_labels import ObjectMetaLabels

        d = dict(src_dict)
        name = d.pop("name", UNSET)

        namespace = d.pop("namespace", UNSET)

        _labels = d.pop("labels", UNSET)
        labels: Union[Unset, ObjectMetaLabels]
        if isinstance(_labels, Unset):
            labels = UNSET
        else:
            labels = ObjectMetaLabels.from_dict(_labels)

        _annotations = d.pop("annotations", UNSET)
        annotations: Union[Unset, ObjectMetaAnnotations]
        if isinstance(_annotations, Unset):
            annotations = UNSET
        else:
            annotations = ObjectMetaAnnotations.from_dict(_annotations)

        object_meta = cls(
            name=name,
            namespace=namespace,
            labels=labels,
            annotations=annotations,
        )

        object_meta.additional_properties = d
        return object_meta

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.file_info import FileInfo


T = TypeVar("T", bound="SuccessFileListResponseData")


@_attrs_define
class SuccessFileListResponseData:
    """
    Attributes:
        entries (Union[Unset, list['FileInfo']]):
    """

    entries: Union[Unset, list["FileInfo"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        entries: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.entries, Unset):
            entries = []
            for entries_item_data in self.entries:
                entries_item = entries_item_data.to_dict()
                entries.append(entries_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if entries is not UNSET:
            field_dict["entries"] = entries

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_info import FileInfo

        d = dict(src_dict)
        entries = []
        _entries = d.pop("entries", UNSET)
        for entries_item_data in _entries or []:
            entries_item = FileInfo.from_dict(entries_item_data)

            entries.append(entries_item)

        success_file_list_response_data = cls(
            entries=entries,
        )

        success_file_list_response_data.additional_properties = d
        return success_file_list_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.process_type import ProcessType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_cmd_context_request import CreateCMDContextRequest
    from ..models.create_context_request_env_vars import CreateContextRequestEnvVars
    from ..models.create_repl_context_request import CreateREPLContextRequest
    from ..models.pty_size import PTYSize


T = TypeVar("T", bound="CreateContextRequest")


@_attrs_define
class CreateContextRequest:
    """
    Attributes:
        type_ (Union[Unset, ProcessType]):
        repl (Union[Unset, CreateREPLContextRequest]):
        cmd (Union[Unset, CreateCMDContextRequest]):
        wait_until_done (Union[Unset, bool]):
        cwd (Union[Unset, str]):
        env_vars (Union[Unset, CreateContextRequestEnvVars]):
        pty_size (Union[Unset, PTYSize]):
        idle_timeout_sec (Union[Unset, int]):
        ttl_sec (Union[Unset, int]):
    """

    type_: Union[Unset, ProcessType] = UNSET
    repl: Union[Unset, "CreateREPLContextRequest"] = UNSET
    cmd: Union[Unset, "CreateCMDContextRequest"] = UNSET
    wait_until_done: Union[Unset, bool] = UNSET
    cwd: Union[Unset, str] = UNSET
    env_vars: Union[Unset, "CreateContextRequestEnvVars"] = UNSET
    pty_size: Union[Unset, "PTYSize"] = UNSET
    idle_timeout_sec: Union[Unset, int] = UNSET
    ttl_sec: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        repl: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.repl, Unset):
            repl = self.repl.to_dict()

        cmd: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.cmd, Unset):
            cmd = self.cmd.to_dict()

        wait_until_done = self.wait_until_done

        cwd = self.cwd

        env_vars: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.env_vars, Unset):
            env_vars = self.env_vars.to_dict()

        pty_size: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.pty_size, Unset):
            pty_size = self.pty_size.to_dict()

        idle_timeout_sec = self.idle_timeout_sec

        ttl_sec = self.ttl_sec

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type_ is not UNSET:
            field_dict["type"] = type_
        if repl is not UNSET:
            field_dict["repl"] = repl
        if cmd is not UNSET:
            field_dict["cmd"] = cmd
        if wait_until_done is not UNSET:
            field_dict["wait_until_done"] = wait_until_done
        if cwd is not UNSET:
            field_dict["cwd"] = cwd
        if env_vars is not UNSET:
            field_dict["env_vars"] = env_vars
        if pty_size is not UNSET:
            field_dict["pty_size"] = pty_size
        if idle_timeout_sec is not UNSET:
            field_dict["idle_timeout_sec"] = idle_timeout_sec
        if ttl_sec is not UNSET:
            field_dict["ttl_sec"] = ttl_sec

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.create_cmd_context_request import CreateCMDContextRequest
        from ..models.create_context_request_env_vars import CreateContextRequestEnvVars
        from ..models.create_repl_context_request import CreateREPLContextRequest
        from ..models.pty_size import PTYSize

        d = dict(src_dict)
        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, ProcessType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ProcessType(_type_)

        _repl = d.pop("repl", UNSET)
        repl: Union[Unset, CreateREPLContextRequest]
        if isinstance(_repl, Unset):
            repl = UNSET
        else:
            repl = CreateREPLContextRequest.from_dict(_repl)

        _cmd = d.pop("cmd", UNSET)
        cmd: Union[Unset, CreateCMDContextRequest]
        if isinstance(_cmd, Unset):
            cmd = UNSET
        else:
            cmd = CreateCMDContextRequest.from_dict(_cmd)

        wait_until_done = d.pop("wait_until_done", UNSET)

        cwd = d.pop("cwd", UNSET)

        _env_vars = d.pop("env_vars", UNSET)
        env_vars: Union[Unset, CreateContextRequestEnvVars]
        if isinstance(_env_vars, Unset):
            env_vars = UNSET
        else:
            env_vars = CreateContextRequestEnvVars.from_dict(_env_vars)

        _pty_size = d.pop("pty_size", UNSET)
        pty_size: Union[Unset, PTYSize]
        if isinstance(_pty_size, Unset):
            pty_size = UNSET
        else:
            pty_size = PTYSize.from_dict(_pty_size)

        idle_timeout_sec = d.pop("idle_timeout_sec", UNSET)

        ttl_sec = d.pop("ttl_sec", UNSET)

        create_context_request = cls(
            type_=type_,
            repl=repl,
            cmd=cmd,
            wait_until_done=wait_until_done,
            cwd=cwd,
            env_vars=env_vars,
            pty_size=pty_size,
            idle_timeout_sec=idle_timeout_sec,
            ttl_sec=ttl_sec,
        )

        create_context_request.additional_properties = d
        return create_context_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.context_response import ContextResponse


T = TypeVar("T", bound="SuccessContextListResponseData")


@_attrs_define
class SuccessContextListResponseData:
    """
    Attributes:
        contexts (Union[Unset, list['ContextResponse']]):
    """

    contexts: Union[Unset, list["ContextResponse"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        contexts: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.contexts, Unset):
            contexts = []
            for contexts_item_data in self.contexts:
                contexts_item = contexts_item_data.to_dict()
                contexts.append(contexts_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if contexts is not UNSET:
            field_dict["contexts"] = contexts

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.context_response import ContextResponse

        d = dict(src_dict)
        contexts = []
        _contexts = d.pop("contexts", UNSET)
        for contexts_item_data in _contexts or []:
            contexts_item = ContextResponse.from_dict(contexts_item_data)

            contexts.append(contexts_item)

        success_context_list_response_data = cls(
            contexts=contexts,
        )

        success_context_list_response_data.additional_properties = d
        return success_context_list_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pre_stop_hook import PreStopHook


T = TypeVar("T", bound="LifecyclePolicy")


@_attrs_define
class LifecyclePolicy:
    """
    Attributes:
        default_ttl (Union[Unset, int]):
        max_ttl (Union[Unset, int]):
        idle_timeout (Union[Unset, int]):
        pre_stop (Union[Unset, PreStopHook]):
    """

    default_ttl: Union[Unset, int] = UNSET
    max_ttl: Union[Unset, int] = UNSET
    idle_timeout: Union[Unset, int] = UNSET
    pre_stop: Union[Unset, "PreStopHook"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        default_ttl = self.default_ttl

        max_ttl = self.max_ttl

        idle_timeout = self.idle_timeout

        pre_stop: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.pre_stop, Unset):
            pre_stop = self.pre_stop.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if default_ttl is not UNSET:
            field_dict["defaultTTL"] = default_ttl
        if max_ttl is not UNSET:
            field_dict["maxTTL"] = max_ttl
        if idle_timeout is not UNSET:
            field_dict["idleTimeout"] = idle_timeout
        if pre_stop is not UNSET:
            field_dict["preStop"] = pre_stop

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.pre_stop_hook import PreStopHook

        d = dict(src_dict)
        default_ttl = d.pop("defaultTTL", UNSET)

        max_ttl = d.pop("maxTTL", UNSET)

        idle_timeout = d.pop("idleTimeout", UNSET)

        _pre_stop = d.pop("preStop", UNSET)
        pre_stop: Union[Unset, PreStopHook]
        if isinstance(_pre_stop, Unset):
            pre_stop = UNSET
        else:
            pre_stop = PreStopHook.from_dict(_pre_stop)

        lifecycle_policy = cls(
            default_ttl=default_ttl,
            max_ttl=max_ttl,
            idle_timeout=idle_timeout,
            pre_stop=pre_stop,
        )

        lifecycle_policy.additional_properties = d
        return lifecycle_policy

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WebhookConfig")


@_attrs_define
class WebhookConfig:
    """
    Attributes:
        url (Union[Unset, str]): Required when webhook is enabled. Target URL that receives event callbacks.
        secret (Union[Unset, str]): Optional. Shared secret used to sign webhook payloads.
        watch_dir (Union[Unset, str]): Optional. When set, procd subscribes to file events under this directory (same
            semantics as the file watch WebSocket API) and emits file.modified events.
    """

    url: Union[Unset, str] = UNSET
    secret: Union[Unset, str] = UNSET
    watch_dir: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        url = self.url

        secret = self.secret

        watch_dir = self.watch_dir

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if url is not UNSET:
            field_dict["url"] = url
        if secret is not UNSET:
            field_dict["secret"] = secret
        if watch_dir is not UNSET:
            field_dict["watch_dir"] = watch_dir

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        url = d.pop("url", UNSET)

        secret = d.pop("secret", UNSET)

        watch_dir = d.pop("watch_dir", UNSET)

        webhook_config = cls(
            url=url,
            secret=secret,
            watch_dir=watch_dir,
        )

        webhook_config.additional_properties = d
        return webhook_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from typing import Optional, TYPE_CHECKING

from sandbox0.apispec.api.sandbox_volumes import get_api_v1_sandboxes_id_sandboxvolumes_status
from sandbox0.apispec.api.sandbox_volumes import post_api_v1_sandboxes_id_sandboxvolumes_mount
from sandbox0.apispec.api.sandbox_volumes import post_api_v1_sandboxes_id_sandboxvolumes_unmount
from sandbox0.apispec.models.mount_request import MountRequest
from sandbox0.apispec.models.mount_status import MountStatus
from sandbox0.apispec.models.success_mount_response import SuccessMountResponse
from sandbox0.apispec.models.success_mount_status_response import SuccessMountStatusResponse
from sandbox0.apispec.models.success_unmounted_response import SuccessUnmountedResponse
from sandbox0.apispec.models.unmount_request import UnmountRequest
from sandbox0.apispec.models.volume_config import VolumeConfig
from sandbox0.apispec.types import UNSET
from sandbox0.response import ensure_data, ensure_model
from sandbox0.sessions import MountSession

if TYPE_CHECKING:
    from sandbox0.sandbox import Sandbox


class SandboxVolumesMixin:
    id: str
    _client: any

    def mount(self: "Sandbox", volume_id: str, mount_point: str, config: Optional[VolumeConfig] = None) -> MountSession:
        request = MountRequest(
            sandboxvolume_id=volume_id,
            mount_point=mount_point,
            volume_config=config if config is not None else UNSET,
        )
        resp = post_api_v1_sandboxes_id_sandboxvolumes_mount.sync_detailed(
            id=self.id,
            client=self._client.api,
            body=request,
        )
        mount_resp = ensure_data(resp, SuccessMountResponse)
        return MountSession(
            volume_id=volume_id,
            mount_point=mount_resp.mount_point,
            mount_session_id=mount_resp.mount_session_id,
            unmount=lambda: self.unmount(volume_id, mount_resp.mount_session_id),
        )

    def unmount(self: "Sandbox", volume_id: str, mount_session_id: str) -> SuccessUnmountedResponse:
        resp = post_api_v1_sandboxes_id_sandboxvolumes_unmount.sync_detailed(
            id=self.id,
            client=self._client.api,
            body=UnmountRequest(sandboxvolume_id=volume_id, mount_session_id=mount_session_id),
        )
        return ensure_model(resp, SuccessUnmountedResponse)

    def mount_status(self: "Sandbox") -> list[MountStatus]:
        resp = get_api_v1_sandboxes_id_sandboxvolumes_status.sync_detailed(id=self.id, client=self._client.api)
        data = ensure_data(resp, SuccessMountStatusResponse)
        mounts = data.mounts
        if mounts.__class__.__name__ == "Unset":
            return []
        return mounts



from __future__ import annotations

from typing import TYPE_CHECKING

from sandbox0.apispec.api.contexts import delete_api_v1_sandboxes_id_contexts_ctx_id
from sandbox0.apispec.api.contexts import get_api_v1_sandboxes_id_contexts
from sandbox0.apispec.api.contexts import get_api_v1_sandboxes_id_contexts_ctx_id
from sandbox0.apispec.api.contexts import get_api_v1_sandboxes_id_contexts_ctx_id_stats
from sandbox0.apispec.api.contexts import post_api_v1_sandboxes_id_contexts
from sandbox0.apispec.api.contexts import post_api_v1_sandboxes_id_contexts_ctx_id_exec
from sandbox0.apispec.api.contexts import post_api_v1_sandboxes_id_contexts_ctx_id_input
from sandbox0.apispec.api.contexts import post_api_v1_sandboxes_id_contexts_ctx_id_resize
from sandbox0.apispec.api.contexts import post_api_v1_sandboxes_id_contexts_ctx_id_restart
from sandbox0.apispec.api.contexts import post_api_v1_sandboxes_id_contexts_ctx_id_signal
from sandbox0.apispec.models.context_exec_response import ContextExecResponse
from sandbox0.apispec.models.context_input_request import ContextInputRequest
from sandbox0.apispec.models.context_response import ContextResponse
from sandbox0.apispec.models.context_stats_response import ContextStatsResponse
from sandbox0.apispec.models.create_context_request import CreateContextRequest
from sandbox0.apispec.models.resize_context_request import ResizeContextRequest
from sandbox0.apispec.models.signal_context_request import SignalContextRequest
from sandbox0.apispec.models.success_context_exec_response import SuccessContextExecResponse
from sandbox0.apispec.models.success_context_list_response import SuccessContextListResponse
from sandbox0.apispec.models.success_context_response import SuccessContextResponse
from sandbox0.apispec.models.success_context_stats_response import SuccessContextStatsResponse
from sandbox0.apispec.models.success_deleted_response import SuccessDeletedResponse
from sandbox0.apispec.models.success_resized_response import SuccessResizedResponse
from sandbox0.apispec.models.success_signaled_response import SuccessSignaledResponse
from sandbox0.apispec.models.success_written_response import SuccessWrittenResponse
from sandbox0.response import ensure_data, ensure_model

if TYPE_CHECKING:
    from sandbox0.sandbox import Sandbox


class SandboxContextsMixin:
    id: str
    _client: any

    def list_contexts(self: "Sandbox") -> list[ContextResponse]:
        resp = get_api_v1_sandboxes_id_contexts.sync_detailed(id=self.id, client=self._client.api)
        data = ensure_data(resp, SuccessContextListResponse)
        contexts = data.contexts
        if contexts.__class__.__name__ == "Unset":
            return []
        return contexts

    # Backward-compatible alias.
    def list_context(self: "Sandbox") -> list[ContextResponse]:
        return self.list_contexts()

    def create_context(self: "Sandbox", request: CreateContextRequest) -> ContextResponse:
        resp = post_api_v1_sandboxes_id_contexts.sync_detailed(id=self.id, client=self._client.api, body=request)
        return ensure_data(resp, SuccessContextResponse)

    def get_context(self: "Sandbox", context_id: str) -> ContextResponse:
        resp = get_api_v1_sandboxes_id_contexts_ctx_id.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
        )
        return ensure_data(resp, SuccessContextResponse)

    def delete_context(self: "Sandbox", context_id: str) -> SuccessDeletedResponse:
        resp = delete_api_v1_sandboxes_id_contexts_ctx_id.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
        )
        return ensure_model(resp, SuccessDeletedResponse)

    def restart_context(self: "Sandbox", context_id: str) -> ContextResponse:
        resp = post_api_v1_sandboxes_id_contexts_ctx_id_restart.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
        )
        return ensure_data(resp, SuccessContextResponse)

    def context_input(self: "Sandbox", context_id: str, input: str) -> SuccessWrittenResponse:
        resp = post_api_v1_sandboxes_id_contexts_ctx_id_input.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
            body=ContextInputRequest(data=input),
        )
        return ensure_model(resp, SuccessWrittenResponse)

    def context_exec(self: "Sandbox", context_id: str, input: str) -> ContextExecResponse:
        resp = post_api_v1_sandboxes_id_contexts_ctx_id_exec.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
            body=ContextInputRequest(data=input),
        )
        return ensure_data(resp, SuccessContextExecResponse)

    def context_resize(self: "Sandbox", context_id: str, rows: int, cols: int) -> SuccessResizedResponse:
        resp = post_api_v1_sandboxes_id_contexts_ctx_id_resize.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
            body=ResizeContextRequest(rows=rows, cols=cols),
        )
        return ensure_model(resp, SuccessResizedResponse)

    def context_signal(self: "Sandbox", context_id: str, signal: str) -> SuccessSignaledResponse:
        resp = post_api_v1_sandboxes_id_contexts_ctx_id_signal.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
            body=SignalContextRequest(signal=signal),
        )
        return ensure_model(resp, SuccessSignaledResponse)

    def context_stats(self: "Sandbox", context_id: str) -> ContextStatsResponse:
        resp = get_api_v1_sandboxes_id_contexts_ctx_id_stats.sync_detailed(
            id=self.id,
            ctx_id=context_id,
            client=self._client.api,
        )
        return ensure_data(resp, SuccessContextStatsResponse)



from __future__ import annotations

from typing import TYPE_CHECKING

from sandbox0.apispec.api.sandboxes import get_api_v1_sandboxes_id_network
from sandbox0.apispec.api.sandboxes import put_api_v1_sandboxes_id_network
from sandbox0.apispec.models.success_sandbox_network_policy_response import SuccessSandboxNetworkPolicyResponse
from sandbox0.apispec.models.tpl_sandbox_network_policy import TplSandboxNetworkPolicy
from sandbox0.response import ensure_data

if TYPE_CHECKING:
    from sandbox0.sandbox import Sandbox


class SandboxNetworkMixin:
    id: str
    _client: any

    def get_network_policy(self: "Sandbox") -> TplSandboxNetworkPolicy:
        resp = get_api_v1_sandboxes_id_network.sync_detailed(id=self.id, client=self._client.api)
        return ensure_data(resp, SuccessSandboxNetworkPolicyResponse)

    def update_network_policy(self: "Sandbox", policy: TplSandboxNetworkPolicy) -> TplSandboxNetworkPolicy:
        resp = put_api_v1_sandboxes_id_network.sync_detailed(id=self.id, client=self._client.api, body=policy)
        return ensure_data(resp, SuccessSandboxNetworkPolicyResponse)



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.create_api_key_request_type import CreateAPIKeyRequestType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateAPIKeyRequest")


@_attrs_define
class CreateAPIKeyRequest:
    """
    Attributes:
        name (str):
        type_ (CreateAPIKeyRequestType):
        roles (Union[Unset, list[str]]):
        expires_in (Union[Unset, str]): 30d, 90d, 180d, 365d, or never
    """

    name: str
    type_: CreateAPIKeyRequestType
    roles: Union[Unset, list[str]] = UNSET
    expires_in: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        type_ = self.type_.value

        roles: Union[Unset, list[str]] = UNSET
        if not isinstance(self.roles, Unset):
            roles = self.roles

        expires_in = self.expires_in

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "type": type_,
            }
        )
        if roles is not UNSET:
            field_dict["roles"] = roles
        if expires_in is not UNSET:
            field_dict["expires_in"] = expires_in

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name")

        type_ = CreateAPIKeyRequestType(d.pop("type"))

        roles = cast(list[str], d.pop("roles", UNSET))

        expires_in = d.pop("expires_in", UNSET)

        create_api_key_request = cls(
            name=name,
            type_=type_,
            roles=roles,
            expires_in=expires_in,
        )

        create_api_key_request.additional_properties = d
        return create_api_key_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.sandbox_template_spec import SandboxTemplateSpec


T = TypeVar("T", bound="Template")


@_attrs_define
class Template:
    """
    Attributes:
        template_id (str):
        scope (str):
        spec (SandboxTemplateSpec):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        team_id (Union[Unset, str]):
        user_id (Union[Unset, str]):
    """

    template_id: str
    scope: str
    spec: "SandboxTemplateSpec"
    created_at: datetime.datetime
    updated_at: datetime.datetime
    team_id: Union[Unset, str] = UNSET
    user_id: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        template_id = self.template_id

        scope = self.scope

        spec = self.spec.to_dict()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        team_id = self.team_id

        user_id = self.user_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "template_id": template_id,
                "scope": scope,
                "spec": spec,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if team_id is not UNSET:
            field_dict["team_id"] = team_id
        if user_id is not UNSET:
            field_dict["user_id"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.sandbox_template_spec import SandboxTemplateSpec

        d = dict(src_dict)
        template_id = d.pop("template_id")

        scope = d.pop("scope")

        spec = SandboxTemplateSpec.from_dict(d.pop("spec"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        team_id = d.pop("team_id", UNSET)

        user_id = d.pop("user_id", UNSET)

        template = cls(
            template_id=template_id,
            scope=scope,
            spec=spec,
            created_at=created_at,
            updated_at=updated_at,
            team_id=team_id,
            user_id=user_id,
        )

        template.additional_properties = d
        return template

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.file_watch_unsubscribe_request_action import (
    FileWatchUnsubscribeRequestAction,
)

T = TypeVar("T", bound="FileWatchUnsubscribeRequest")


@_attrs_define
class FileWatchUnsubscribeRequest:
    """
    Attributes:
        action (FileWatchUnsubscribeRequestAction):
        watch_id (str):
    """

    action: FileWatchUnsubscribeRequestAction
    watch_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        action = self.action.value

        watch_id = self.watch_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "action": action,
                "watch_id": watch_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        action = FileWatchUnsubscribeRequestAction(d.pop("action"))

        watch_id = d.pop("watch_id")

        file_watch_unsubscribe_request = cls(
            action=action,
            watch_id=watch_id,
        )

        file_watch_unsubscribe_request.additional_properties = d
        return file_watch_unsubscribe_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.file_content_response_encoding import FileContentResponseEncoding
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileContentResponse")


@_attrs_define
class FileContentResponse:
    """
    Attributes:
        content (Union[Unset, str]):
        encoding (Union[Unset, FileContentResponseEncoding]):
    """

    content: Union[Unset, str] = UNSET
    encoding: Union[Unset, FileContentResponseEncoding] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        content = self.content

        encoding: Union[Unset, str] = UNSET
        if not isinstance(self.encoding, Unset):
            encoding = self.encoding.value

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if content is not UNSET:
            field_dict["content"] = content
        if encoding is not UNSET:
            field_dict["encoding"] = encoding

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        content = d.pop("content", UNSET)

        _encoding = d.pop("encoding", UNSET)
        encoding: Union[Unset, FileContentResponseEncoding]
        if isinstance(_encoding, Unset):
            encoding = UNSET
        else:
            encoding = FileContentResponseEncoding(_encoding)

        file_content_response = cls(
            content=content,
            encoding=encoding,
        )

        file_content_response.additional_properties = d
        return file_content_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="APIKey")


@_attrs_define
class APIKey:
    """
    Attributes:
        id (str):
        team_id (str):
        created_by (str):
        name (str):
        type_ (str):
        roles (list[str]):
        is_active (bool):
        expires_at (datetime.datetime):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        key_value (Union[Unset, str]):
        user_id (Union[None, Unset, str]):
        last_used_at (Union[Unset, datetime.datetime]):
        usage_count (Union[Unset, int]):
    """

    id: str
    team_id: str
    created_by: str
    name: str
    type_: str
    roles: list[str]
    is_active: bool
    expires_at: datetime.datetime
    created_at: datetime.datetime
    updated_at: datetime.datetime
    key_value: Union[Unset, str] = UNSET
    user_id: Union[None, Unset, str] = UNSET
    last_used_at: Union[Unset, datetime.datetime] = UNSET
    usage_count: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        team_id = self.team_id

        created_by = self.created_by

        name = self.name

        type_ = self.type_

        roles = self.roles

        is_active = self.is_active

        expires_at = self.expires_at.isoformat()

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        key_value = self.key_value

        user_id: Union[None, Unset, str]
        if isinstance(self.user_id, Unset):
            user_id = UNSET
        else:
            user_id = self.user_id

        last_used_at: Union[Unset, str] = UNSET
        if not isinstance(self.last_used_at, Unset):
            last_used_at = self.last_used_at.isoformat()

        usage_count = self.usage_count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "team_id": team_id,
                "created_by": created_by,
                "name": name,
                "type": type_,
                "roles": roles,
                "is_active": is_active,
                "expires_at": expires_at,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if key_value is not UNSET:
            field_dict["key_value"] = key_value
        if user_id is not UNSET:
            field_dict["user_id"] = user_id
        if last_used_at is not UNSET:
            field_dict["last_used_at"] = last_used_at
        if usage_count is not UNSET:
            field_dict["usage_count"] = usage_count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        team_id = d.pop("team_id")

        created_by = d.pop("created_by")

        name = d.pop("name")

        type_ = d.pop("type")

        roles = cast(list[str], d.pop("roles"))

        is_active = d.pop("is_active")

        expires_at = isoparse(d.pop("expires_at"))

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        key_value = d.pop("key_value", UNSET)

        def _parse_user_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        user_id = _parse_user_id(d.pop("user_id", UNSET))

        _last_used_at = d.pop("last_used_at", UNSET)
        last_used_at: Union[Unset, datetime.datetime]
        if isinstance(_last_used_at, Unset):
            last_used_at = UNSET
        else:
            last_used_at = isoparse(_last_used_at)

        usage_count = d.pop("usage_count", UNSET)

        api_key = cls(
            id=id,
            team_id=team_id,
            created_by=created_by,
            name=name,
            type_=type_,
            roles=roles,
            is_active=is_active,
            expires_at=expires_at,
            created_at=created_at,
            updated_at=updated_at,
            key_value=key_value,
            user_id=user_id,
            last_used_at=last_used_at,
            usage_count=usage_count,
        )

        api_key.additional_properties = d
        return api_key

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.capabilities import Capabilities


T = TypeVar("T", bound="SecurityContext")


@_attrs_define
class SecurityContext:
    """
    Attributes:
        capabilities (Union[Unset, Capabilities]):
        run_as_user (Union[Unset, int]):
        run_as_group (Union[Unset, int]):
    """

    capabilities: Union[Unset, "Capabilities"] = UNSET
    run_as_user: Union[Unset, int] = UNSET
    run_as_group: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        capabilities: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.capabilities, Unset):
            capabilities = self.capabilities.to_dict()

        run_as_user = self.run_as_user

        run_as_group = self.run_as_group

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if capabilities is not UNSET:
            field_dict["capabilities"] = capabilities
        if run_as_user is not UNSET:
            field_dict["runAsUser"] = run_as_user
        if run_as_group is not UNSET:
            field_dict["runAsGroup"] = run_as_group

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.capabilities import Capabilities

        d = dict(src_dict)
        _capabilities = d.pop("capabilities", UNSET)
        capabilities: Union[Unset, Capabilities]
        if isinstance(_capabilities, Unset):
            capabilities = UNSET
        else:
            capabilities = Capabilities.from_dict(_capabilities)

        run_as_user = d.pop("runAsUser", UNSET)

        run_as_group = d.pop("runAsGroup", UNSET)

        security_context = cls(
            capabilities=capabilities,
            run_as_user=run_as_user,
            run_as_group=run_as_group,
        )

        security_context.additional_properties = d
        return security_context

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.repl_ready_mode import REPLReadyMode
from ..types import UNSET, Unset

T = TypeVar("T", bound="REPLReadyConfig")


@_attrs_define
class REPLReadyConfig:
    """
    Attributes:
        mode (Union[Unset, REPLReadyMode]):
        token (Union[Unset, str]):
        startup_delay_ms (Union[Unset, int]):
    """

    mode: Union[Unset, REPLReadyMode] = UNSET
    token: Union[Unset, str] = UNSET
    startup_delay_ms: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        mode: Union[Unset, str] = UNSET
        if not isinstance(self.mode, Unset):
            mode = self.mode.value

        token = self.token

        startup_delay_ms = self.startup_delay_ms

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if mode is not UNSET:
            field_dict["mode"] = mode
        if token is not UNSET:
            field_dict["token"] = token
        if startup_delay_ms is not UNSET:
            field_dict["startup_delay_ms"] = startup_delay_ms

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        _mode = d.pop("mode", UNSET)
        mode: Union[Unset, REPLReadyMode]
        if isinstance(_mode, Unset):
            mode = UNSET
        else:
            mode = REPLReadyMode(_mode)

        token = d.pop("token", UNSET)

        startup_delay_ms = d.pop("startup_delay_ms", UNSET)

        repl_ready_config = cls(
            mode=mode,
            token=token,
            startup_delay_ms=startup_delay_ms,
        )

        repl_ready_config.additional_properties = d
        return repl_ready_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.affinity import Affinity
    from ..models.pod_spec_override_node_selector import PodSpecOverrideNodeSelector
    from ..models.toleration import Toleration


T = TypeVar("T", bound="PodSpecOverride")


@_attrs_define
class PodSpecOverride:
    """
    Attributes:
        node_selector (Union[Unset, PodSpecOverrideNodeSelector]):
        affinity (Union[Unset, Affinity]):
        tolerations (Union[Unset, list['Toleration']]):
        service_account_name (Union[Unset, str]):
    """

    node_selector: Union[Unset, "PodSpecOverrideNodeSelector"] = UNSET
    affinity: Union[Unset, "Affinity"] = UNSET
    tolerations: Union[Unset, list["Toleration"]] = UNSET
    service_account_name: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        node_selector: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.node_selector, Unset):
            node_selector = self.node_selector.to_dict()

        affinity: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.affinity, Unset):
            affinity = self.affinity.to_dict()

        tolerations: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.tolerations, Unset):
            tolerations = []
            for tolerations_item_data in self.tolerations:
                tolerations_item = tolerations_item_data.to_dict()
                tolerations.append(tolerations_item)

        service_account_name = self.service_account_name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if node_selector is not UNSET:
            field_dict["nodeSelector"] = node_selector
        if affinity is not UNSET:
            field_dict["affinity"] = affinity
        if tolerations is not UNSET:
            field_dict["tolerations"] = tolerations
        if service_account_name is not UNSET:
            field_dict["serviceAccountName"] = service_account_name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.affinity import Affinity
        from ..models.pod_spec_override_node_selector import PodSpecOverrideNodeSelector
        from ..models.toleration import Toleration

        d = dict(src_dict)
        _node_selector = d.pop("nodeSelector", UNSET)
        node_selector: Union[Unset, PodSpecOverrideNodeSelector]
        if isinstance(_node_selector, Unset):
            node_selector = UNSET
        else:
            node_selector = PodSpecOverrideNodeSelector.from_dict(_node_selector)

        _affinity = d.pop("affinity", UNSET)
        affinity: Union[Unset, Affinity]
        if isinstance(_affinity, Unset):
            affinity = UNSET
        else:
            affinity = Affinity.from_dict(_affinity)

        tolerations = []
        _tolerations = d.pop("tolerations", UNSET)
        for tolerations_item_data in _tolerations or []:
            tolerations_item = Toleration.from_dict(tolerations_item_data)

            tolerations.append(tolerations_item)

        service_account_name = d.pop("serviceAccountName", UNSET)

        pod_spec_override = cls(
            node_selector=node_selector,
            affinity=affinity,
            tolerations=tolerations,
            service_account_name=service_account_name,
        )

        pod_spec_override.additional_properties = d
        return pod_spec_override

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.exposed_port_config import ExposedPortConfig


T = TypeVar("T", bound="UpdateExposedPortsRequest")


@_attrs_define
class UpdateExposedPortsRequest:
    """
    Attributes:
        ports (list['ExposedPortConfig']):
    """

    ports: list["ExposedPortConfig"]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        ports = []
        for ports_item_data in self.ports:
            ports_item = ports_item_data.to_dict()
            ports.append(ports_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "ports": ports,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.exposed_port_config import ExposedPortConfig

        d = dict(src_dict)
        ports = []
        _ports = d.pop("ports")
        for ports_item_data in _ports:
            ports_item = ExposedPortConfig.from_dict(ports_item_data)

            ports.append(ports_item)

        update_exposed_ports_request = cls(
            ports=ports,
        )

        update_exposed_ports_request.additional_properties = d
        return update_exposed_ports_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from sandbox0.client import Client
from sandbox0.errors import APIError
from sandbox0.models import (
    CmdResult,
    ExposedPort,
    ExposedPortsResponse,
    FileWatchResponse,
    RunResult,
    StreamInput,
    StreamOutput,
)
from sandbox0.resources import Sandboxes, Volumes
from sandbox0.sandbox import CmdOptions, RunOptions, Sandbox
from sandbox0.sessions import MountSession, SandboxSession, VolumeSession

__all__ = [
    "APIError",
    "Client",
    "CmdOptions",
    "CmdResult",
    "ExposedPort",
    "ExposedPortsResponse",
    "FileWatchResponse",
    "MountSession",
    "RunOptions",
    "RunResult",
    "Sandboxes",
    "Sandbox",
    "SandboxSession",
    "StreamInput",
    "StreamOutput",
    "Volumes",
    "VolumeSession",
]



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.identity import Identity


T = TypeVar("T", bound="SuccessIdentityListResponseData")


@_attrs_define
class SuccessIdentityListResponseData:
    """
    Attributes:
        identities (Union[Unset, list['Identity']]):
    """

    identities: Union[Unset, list["Identity"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        identities: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.identities, Unset):
            identities = []
            for identities_item_data in self.identities:
                identities_item = identities_item_data.to_dict()
                identities.append(identities_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if identities is not UNSET:
            field_dict["identities"] = identities

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.identity import Identity

        d = dict(src_dict)
        identities = []
        _identities = d.pop("identities", UNSET)
        for identities_item_data in _identities or []:
            identities_item = Identity.from_dict(identities_item_data)

            identities.append(identities_item)

        success_identity_list_response_data = cls(
            identities=identities,
        )

        success_identity_list_response_data.additional_properties = d
        return success_identity_list_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="PortSpec")


@_attrs_define
class PortSpec:
    """
    Attributes:
        port (int):
        protocol (Union[Unset, str]):
        end_port (Union[Unset, int]):
    """

    port: int
    protocol: Union[Unset, str] = UNSET
    end_port: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        port = self.port

        protocol = self.protocol

        end_port = self.end_port

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "port": port,
            }
        )
        if protocol is not UNSET:
            field_dict["protocol"] = protocol
        if end_port is not UNSET:
            field_dict["endPort"] = end_port

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        port = d.pop("port")

        protocol = d.pop("protocol", UNSET)

        end_port = d.pop("endPort", UNSET)

        port_spec = cls(
            port=port,
            protocol=protocol,
            end_port=end_port,
        )

        port_spec.additional_properties = d
        return port_spec

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="RegistryCredentials")


@_attrs_define
class RegistryCredentials:
    """
    Attributes:
        provider (str):
        registry (str):
        username (str):
        password (str):
        expires_at (Union[Unset, datetime.datetime]):
    """

    provider: str
    registry: str
    username: str
    password: str
    expires_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        provider = self.provider

        registry = self.registry

        username = self.username

        password = self.password

        expires_at: Union[Unset, str] = UNSET
        if not isinstance(self.expires_at, Unset):
            expires_at = self.expires_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "provider": provider,
                "registry": registry,
                "username": username,
                "password": password,
            }
        )
        if expires_at is not UNSET:
            field_dict["expiresAt"] = expires_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        provider = d.pop("provider")

        registry = d.pop("registry")

        username = d.pop("username")

        password = d.pop("password")

        _expires_at = d.pop("expiresAt", UNSET)
        expires_at: Union[Unset, datetime.datetime]
        if isinstance(_expires_at, Unset):
            expires_at = UNSET
        else:
            expires_at = isoparse(_expires_at)

        registry_credentials = cls(
            provider=provider,
            registry=registry,
            username=username,
            password=password,
            expires_at=expires_at,
        )

        registry_credentials.additional_properties = d
        return registry_credentials

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from typing import Optional, TYPE_CHECKING

from sandbox0.apispec.api.sandboxes import delete_api_v1_sandboxes_id
from sandbox0.apispec.api.sandboxes import get_api_v1_sandboxes_id
from sandbox0.apispec.api.sandboxes import get_api_v1_sandboxes_id_status
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes_id_pause
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes_id_refresh
from sandbox0.apispec.api.sandboxes import post_api_v1_sandboxes_id_resume
from sandbox0.apispec.api.sandboxes import put_api_v1_sandboxes_id
from sandbox0.apispec.models.claim_request import ClaimRequest
from sandbox0.apispec.models.claim_response import ClaimResponse
from sandbox0.apispec.models.refresh_request import RefreshRequest
from sandbox0.apispec.models.refresh_response import RefreshResponse
from sandbox0.apispec.models.resume_sandbox_response import ResumeSandboxResponse
from sandbox0.apispec.models.pause_sandbox_response import PauseSandboxResponse
from sandbox0.apispec.models.sandbox import Sandbox as APISandbox
from sandbox0.apispec.models.sandbox_config import SandboxConfig
from sandbox0.apispec.models.sandbox_status import SandboxStatus
from sandbox0.apispec.models.sandbox_update_request import SandboxUpdateRequest
from sandbox0.apispec.models.success_claim_response import SuccessClaimResponse
from sandbox0.apispec.models.success_message_response import SuccessMessageResponse
from sandbox0.apispec.models.success_pause_sandbox_response import SuccessPauseSandboxResponse
from sandbox0.apispec.models.success_refresh_response import SuccessRefreshResponse
from sandbox0.apispec.models.success_resume_sandbox_response import SuccessResumeSandboxResponse
from sandbox0.apispec.models.success_sandbox_response import SuccessSandboxResponse
from sandbox0.apispec.models.success_sandbox_status_response import SuccessSandboxStatusResponse
from sandbox0.response import ensure_data, ensure_model

if TYPE_CHECKING:
    from sandbox0.client import Client
    from sandbox0.sandbox import Sandbox


class ClientSandboxesMixin:
    _api: any

    def claim_sandbox(self: "Client", template: str, config: Optional[SandboxConfig] = None) -> "Sandbox":
        request = ClaimRequest(template=template, config=config) if config is not None else ClaimRequest(template=template)
        resp = post_api_v1_sandboxes.sync_detailed(client=self._api, body=request)
        data = ensure_data(resp, SuccessClaimResponse)
        from sandbox0.sandbox import Sandbox

        return Sandbox(
            id=data.sandbox_id,
            client=self,
            template=data.template,
            cluster_id=None if data.cluster_id.__class__.__name__ == "Unset" else data.cluster_id,
            pod_name=data.pod_name,
            status=data.status,
        )

    def get_sandbox(self: "Client", sandbox_id: str) -> APISandbox:
        resp = get_api_v1_sandboxes_id.sync_detailed(id=sandbox_id, client=self._api)
        return ensure_data(resp, SuccessSandboxResponse)

    def update_sandbox(self: "Client", sandbox_id: str, request: SandboxUpdateRequest) -> APISandbox:
        resp = put_api_v1_sandboxes_id.sync_detailed(id=sandbox_id, client=self._api, body=request)
        return ensure_data(resp, SuccessSandboxResponse)

    def delete_sandbox(self: "Client", sandbox_id: str) -> SuccessMessageResponse:
        resp = delete_api_v1_sandboxes_id.sync_detailed(id=sandbox_id, client=self._api)
        return ensure_model(resp, SuccessMessageResponse)

    def status_sandbox(self: "Client", sandbox_id: str) -> SandboxStatus:
        resp = get_api_v1_sandboxes_id_status.sync_detailed(id=sandbox_id, client=self._api)
        return ensure_data(resp, SuccessSandboxStatusResponse)

    def pause_sandbox(self: "Client", sandbox_id: str) -> PauseSandboxResponse:
        resp = post_api_v1_sandboxes_id_pause.sync_detailed(id=sandbox_id, client=self._api)
        return ensure_data(resp, SuccessPauseSandboxResponse)

    def resume_sandbox(self: "Client", sandbox_id: str) -> ResumeSandboxResponse:
        resp = post_api_v1_sandboxes_id_resume.sync_detailed(id=sandbox_id, client=self._api)
        return ensure_data(resp, SuccessResumeSandboxResponse)

    def refresh_sandbox(self: "Client", sandbox_id: str, request: Optional[RefreshRequest] = None) -> RefreshResponse:
        body = request if request is not None else RefreshRequest()
        resp = post_api_v1_sandboxes_id_refresh.sync_detailed(id=sandbox_id, client=self._api, body=body)
        return ensure_data(resp, SuccessRefreshResponse)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.pod_affinity_term import PodAffinityTerm


T = TypeVar("T", bound="WeightedPodAffinityTerm")


@_attrs_define
class WeightedPodAffinityTerm:
    """
    Attributes:
        weight (int):
        pod_affinity_term (PodAffinityTerm):
    """

    weight: int
    pod_affinity_term: "PodAffinityTerm"
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        weight = self.weight

        pod_affinity_term = self.pod_affinity_term.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "weight": weight,
                "podAffinityTerm": pod_affinity_term,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.pod_affinity_term import PodAffinityTerm

        d = dict(src_dict)
        weight = d.pop("weight")

        pod_affinity_term = PodAffinityTerm.from_dict(d.pop("podAffinityTerm"))

        weighted_pod_affinity_term = cls(
            weight=weight,
            pod_affinity_term=pod_affinity_term,
        )

        weighted_pod_affinity_term.additional_properties = d
        return weighted_pod_affinity_term

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="TeamMember")


@_attrs_define
class TeamMember:
    """
    Attributes:
        id (str):
        team_id (str):
        user_id (str):
        role (str):
        joined_at (datetime.datetime):
    """

    id: str
    team_id: str
    user_id: str
    role: str
    joined_at: datetime.datetime
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        team_id = self.team_id

        user_id = self.user_id

        role = self.role

        joined_at = self.joined_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "team_id": team_id,
                "user_id": user_id,
                "role": role,
                "joined_at": joined_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        team_id = d.pop("team_id")

        user_id = d.pop("user_id")

        role = d.pop("role")

        joined_at = isoparse(d.pop("joined_at"))

        team_member = cls(
            id=id,
            team_id=team_id,
            user_id=user_id,
            role=role,
            joined_at=joined_at,
        )

        team_member.additional_properties = d
        return team_member

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ClaimResponse")


@_attrs_define
class ClaimResponse:
    """
    Attributes:
        sandbox_id (str):
        status (str):
        pod_name (str):
        template (str):
        cluster_id (Union[None, Unset, str]):
    """

    sandbox_id: str
    status: str
    pod_name: str
    template: str
    cluster_id: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        sandbox_id = self.sandbox_id

        status = self.status

        pod_name = self.pod_name

        template = self.template

        cluster_id: Union[None, Unset, str]
        if isinstance(self.cluster_id, Unset):
            cluster_id = UNSET
        else:
            cluster_id = self.cluster_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sandbox_id": sandbox_id,
                "status": status,
                "pod_name": pod_name,
                "template": template,
            }
        )
        if cluster_id is not UNSET:
            field_dict["cluster_id"] = cluster_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        sandbox_id = d.pop("sandbox_id")

        status = d.pop("status")

        pod_name = d.pop("pod_name")

        template = d.pop("template")

        def _parse_cluster_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        cluster_id = _parse_cluster_id(d.pop("cluster_id", UNSET))

        claim_response = cls(
            sandbox_id=sandbox_id,
            status=status,
            pod_name=pod_name,
            template=template,
            cluster_id=cluster_id,
        )

        claim_response.additional_properties = d
        return claim_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.env_var import EnvVar
    from ..models.resource_quota import ResourceQuota
    from ..models.security_context import SecurityContext


T = TypeVar("T", bound="ContainerSpec")


@_attrs_define
class ContainerSpec:
    """
    Attributes:
        image (str):
        resources (ResourceQuota):
        image_pull_policy (Union[Unset, str]):
        env (Union[Unset, list['EnvVar']]):
        security_context (Union[Unset, SecurityContext]):
    """

    image: str
    resources: "ResourceQuota"
    image_pull_policy: Union[Unset, str] = UNSET
    env: Union[Unset, list["EnvVar"]] = UNSET
    security_context: Union[Unset, "SecurityContext"] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        image = self.image

        resources = self.resources.to_dict()

        image_pull_policy = self.image_pull_policy

        env: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.env, Unset):
            env = []
            for env_item_data in self.env:
                env_item = env_item_data.to_dict()
                env.append(env_item)

        security_context: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.security_context, Unset):
            security_context = self.security_context.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "image": image,
                "resources": resources,
            }
        )
        if image_pull_policy is not UNSET:
            field_dict["imagePullPolicy"] = image_pull_policy
        if env is not UNSET:
            field_dict["env"] = env
        if security_context is not UNSET:
            field_dict["securityContext"] = security_context

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.env_var import EnvVar
        from ..models.resource_quota import ResourceQuota
        from ..models.security_context import SecurityContext

        d = dict(src_dict)
        image = d.pop("image")

        resources = ResourceQuota.from_dict(d.pop("resources"))

        image_pull_policy = d.pop("imagePullPolicy", UNSET)

        env = []
        _env = d.pop("env", UNSET)
        for env_item_data in _env or []:
            env_item = EnvVar.from_dict(env_item_data)

            env.append(env_item)

        _security_context = d.pop("securityContext", UNSET)
        security_context: Union[Unset, SecurityContext]
        if isinstance(_security_context, Unset):
            security_context = UNSET
        else:
            security_context = SecurityContext.from_dict(_security_context)

        container_spec = cls(
            image=image,
            resources=resources,
            image_pull_policy=image_pull_policy,
            env=env,
            security_context=security_context,
        )

        container_spec.additional_properties = d
        return container_spec

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.pod_affinity_term import PodAffinityTerm
    from ..models.weighted_pod_affinity_term import WeightedPodAffinityTerm


T = TypeVar("T", bound="PodAffinity")


@_attrs_define
class PodAffinity:
    """
    Attributes:
        required_during_scheduling_ignored_during_execution (Union[Unset, list['PodAffinityTerm']]):
        preferred_during_scheduling_ignored_during_execution (Union[Unset, list['WeightedPodAffinityTerm']]):
    """

    required_during_scheduling_ignored_during_execution: Union[
        Unset, list["PodAffinityTerm"]
    ] = UNSET
    preferred_during_scheduling_ignored_during_execution: Union[
        Unset, list["WeightedPodAffinityTerm"]
    ] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        required_during_scheduling_ignored_during_execution: Union[
            Unset, list[dict[str, Any]]
        ] = UNSET
        if not isinstance(
            self.required_during_scheduling_ignored_during_execution, Unset
        ):
            required_during_scheduling_ignored_during_execution = []
            for (
                required_during_scheduling_ignored_during_execution_item_data
            ) in self.required_during_scheduling_ignored_during_execution:
                required_during_scheduling_ignored_during_execution_item = required_during_scheduling_ignored_during_execution_item_data.to_dict()
                required_during_scheduling_ignored_during_execution.append(
                    required_during_scheduling_ignored_during_execution_item
                )

        preferred_during_scheduling_ignored_during_execution: Union[
            Unset, list[dict[str, Any]]
        ] = UNSET
        if not isinstance(
            self.preferred_during_scheduling_ignored_during_execution, Unset
        ):
            preferred_during_scheduling_ignored_during_execution = []
            for (
                preferred_during_scheduling_ignored_during_execution_item_data
            ) in self.preferred_during_scheduling_ignored_during_execution:
                preferred_during_scheduling_ignored_during_execution_item = preferred_during_scheduling_ignored_during_execution_item_data.to_dict()
                preferred_during_scheduling_ignored_during_execution.append(
                    preferred_during_scheduling_ignored_during_execution_item
                )

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if required_during_scheduling_ignored_during_execution is not UNSET:
            field_dict["requiredDuringSchedulingIgnoredDuringExecution"] = (
                required_during_scheduling_ignored_during_execution
            )
        if preferred_during_scheduling_ignored_during_execution is not UNSET:
            field_dict["preferredDuringSchedulingIgnoredDuringExecution"] = (
                preferred_during_scheduling_ignored_during_execution
            )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.pod_affinity_term import PodAffinityTerm
        from ..models.weighted_pod_affinity_term import WeightedPodAffinityTerm

        d = dict(src_dict)
        required_during_scheduling_ignored_during_execution = []
        _required_during_scheduling_ignored_during_execution = d.pop(
            "requiredDuringSchedulingIgnoredDuringExecution", UNSET
        )
        for required_during_scheduling_ignored_during_execution_item_data in (
            _required_during_scheduling_ignored_during_execution or []
        ):
            required_during_scheduling_ignored_during_execution_item = (
                PodAffinityTerm.from_dict(
                    required_during_scheduling_ignored_during_execution_item_data
                )
            )

            required_during_scheduling_ignored_during_execution.append(
                required_during_scheduling_ignored_during_execution_item
            )

        preferred_during_scheduling_ignored_during_execution = []
        _preferred_during_scheduling_ignored_during_execution = d.pop(
            "preferredDuringSchedulingIgnoredDuringExecution", UNSET
        )
        for preferred_during_scheduling_ignored_during_execution_item_data in (
            _preferred_during_scheduling_ignored_during_execution or []
        ):
            preferred_during_scheduling_ignored_during_execution_item = (
                WeightedPodAffinityTerm.from_dict(
                    preferred_during_scheduling_ignored_during_execution_item_data
                )
            )

            preferred_during_scheduling_ignored_during_execution.append(
                preferred_during_scheduling_ignored_during_execution_item
            )

        pod_affinity = cls(
            required_during_scheduling_ignored_during_execution=required_during_scheduling_ignored_during_execution,
            preferred_during_scheduling_ignored_during_execution=preferred_during_scheduling_ignored_during_execution,
        )

        pod_affinity.additional_properties = d
        return pod_affinity

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.process_type import ProcessType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.context_response_env_vars import ContextResponseEnvVars


T = TypeVar("T", bound="ContextResponse")


@_attrs_define
class ContextResponse:
    """
    Attributes:
        id (str):
        type_ (ProcessType):
        running (bool):
        paused (bool):
        created_at (str):
        language (Union[Unset, str]):
        cwd (Union[Unset, str]):
        env_vars (Union[Unset, ContextResponseEnvVars]):
        output_raw (Union[Unset, str]): Raw PTY output for CMD contexts with wait=true, may contain terminal control
            characters
    """

    id: str
    type_: ProcessType
    running: bool
    paused: bool
    created_at: str
    language: Union[Unset, str] = UNSET
    cwd: Union[Unset, str] = UNSET
    env_vars: Union[Unset, "ContextResponseEnvVars"] = UNSET
    output_raw: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        type_ = self.type_.value

        running = self.running

        paused = self.paused

        created_at = self.created_at

        language = self.language

        cwd = self.cwd

        env_vars: Union[Unset, dict[str, Any]] = UNSET
        if not isinstance(self.env_vars, Unset):
            env_vars = self.env_vars.to_dict()

        output_raw = self.output_raw

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "type": type_,
                "running": running,
                "paused": paused,
                "created_at": created_at,
            }
        )
        if language is not UNSET:
            field_dict["language"] = language
        if cwd is not UNSET:
            field_dict["cwd"] = cwd
        if env_vars is not UNSET:
            field_dict["env_vars"] = env_vars
        if output_raw is not UNSET:
            field_dict["output_raw"] = output_raw

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.context_response_env_vars import ContextResponseEnvVars

        d = dict(src_dict)
        id = d.pop("id")

        type_ = ProcessType(d.pop("type"))

        running = d.pop("running")

        paused = d.pop("paused")

        created_at = d.pop("created_at")

        language = d.pop("language", UNSET)

        cwd = d.pop("cwd", UNSET)

        _env_vars = d.pop("env_vars", UNSET)
        env_vars: Union[Unset, ContextResponseEnvVars]
        if isinstance(_env_vars, Unset):
            env_vars = UNSET
        else:
            env_vars = ContextResponseEnvVars.from_dict(_env_vars)

        output_raw = d.pop("output_raw", UNSET)

        context_response = cls(
            id=id,
            type_=type_,
            running=running,
            paused=paused,
            created_at=created_at,
            language=language,
            cwd=cwd,
            env_vars=env_vars,
            output_raw=output_raw,
        )

        context_response.additional_properties = d
        return context_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.context_web_socket_input_type import ContextWebSocketInputType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContextWebSocketInput")


@_attrs_define
class ContextWebSocketInput:
    """
    Attributes:
        type_ (ContextWebSocketInputType):
        data (Union[Unset, str]):
        request_id (Union[Unset, str]):
    """

    type_: ContextWebSocketInputType
    data: Union[Unset, str] = UNSET
    request_id: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_ = self.type_.value

        data = self.data

        request_id = self.request_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data
        if request_id is not UNSET:
            field_dict["request_id"] = request_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        type_ = ContextWebSocketInputType(d.pop("type"))

        data = d.pop("data", UNSET)

        request_id = d.pop("request_id", UNSET)

        context_web_socket_input = cls(
            type_=type_,
            data=data,
            request_id=request_id,
        )

        context_web_socket_input.additional_properties = d
        return context_web_socket_input

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="VolumeConfig")


@_attrs_define
class VolumeConfig:
    """
    Attributes:
        cache_size (Union[Unset, str]):
        prefetch (Union[Unset, int]):
        buffer_size (Union[Unset, str]):
        writeback (Union[Unset, bool]):
    """

    cache_size: Union[Unset, str] = UNSET
    prefetch: Union[Unset, int] = UNSET
    buffer_size: Union[Unset, str] = UNSET
    writeback: Union[Unset, bool] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        cache_size = self.cache_size

        prefetch = self.prefetch

        buffer_size = self.buffer_size

        writeback = self.writeback

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cache_size is not UNSET:
            field_dict["cache_size"] = cache_size
        if prefetch is not UNSET:
            field_dict["prefetch"] = prefetch
        if buffer_size is not UNSET:
            field_dict["buffer_size"] = buffer_size
        if writeback is not UNSET:
            field_dict["writeback"] = writeback

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        cache_size = d.pop("cache_size", UNSET)

        prefetch = d.pop("prefetch", UNSET)

        buffer_size = d.pop("buffer_size", UNSET)

        writeback = d.pop("writeback", UNSET)

        volume_config = cls(
            cache_size=cache_size,
            prefetch=prefetch,
            buffer_size=buffer_size,
            writeback=writeback,
        )

        volume_config.additional_properties = d
        return volume_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.file_info_type import FileInfoType
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileInfo")


@_attrs_define
class FileInfo:
    """
    Attributes:
        name (Union[Unset, str]):
        path (Union[Unset, str]):
        type_ (Union[Unset, FileInfoType]):
        size (Union[Unset, int]):
        mode (Union[Unset, str]):
        mod_time (Union[Unset, datetime.datetime]):
        is_link (Union[Unset, bool]):
        link_target (Union[Unset, str]):
    """

    name: Union[Unset, str] = UNSET
    path: Union[Unset, str] = UNSET
    type_: Union[Unset, FileInfoType] = UNSET
    size: Union[Unset, int] = UNSET
    mode: Union[Unset, str] = UNSET
    mod_time: Union[Unset, datetime.datetime] = UNSET
    is_link: Union[Unset, bool] = UNSET
    link_target: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        path = self.path

        type_: Union[Unset, str] = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        size = self.size

        mode = self.mode

        mod_time: Union[Unset, str] = UNSET
        if not isinstance(self.mod_time, Unset):
            mod_time = self.mod_time.isoformat()

        is_link = self.is_link

        link_target = self.link_target

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if path is not UNSET:
            field_dict["path"] = path
        if type_ is not UNSET:
            field_dict["type"] = type_
        if size is not UNSET:
            field_dict["size"] = size
        if mode is not UNSET:
            field_dict["mode"] = mode
        if mod_time is not UNSET:
            field_dict["mod_time"] = mod_time
        if is_link is not UNSET:
            field_dict["is_link"] = is_link
        if link_target is not UNSET:
            field_dict["link_target"] = link_target

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name", UNSET)

        path = d.pop("path", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: Union[Unset, FileInfoType]
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = FileInfoType(_type_)

        size = d.pop("size", UNSET)

        mode = d.pop("mode", UNSET)

        _mod_time = d.pop("mod_time", UNSET)
        mod_time: Union[Unset, datetime.datetime]
        if isinstance(_mod_time, Unset):
            mod_time = UNSET
        else:
            mod_time = isoparse(_mod_time)

        is_link = d.pop("is_link", UNSET)

        link_target = d.pop("link_target", UNSET)

        file_info = cls(
            name=name,
            path=path,
            type_=type_,
            size=size,
            mode=mode,
            mod_time=mod_time,
            is_link=is_link,
            link_target=link_target,
        )

        file_info.additional_properties = d
        return file_info

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ResourceQuota")


@_attrs_define
class ResourceQuota:
    """
    Attributes:
        cpu (Union[Unset, str]):
        memory (Union[Unset, str]):
    """

    cpu: Union[Unset, str] = UNSET
    memory: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        cpu = self.cpu

        memory = self.memory

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cpu is not UNSET:
            field_dict["cpu"] = cpu
        if memory is not UNSET:
            field_dict["memory"] = memory

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        cpu = d.pop("cpu", UNSET)

        memory = d.pop("memory", UNSET)

        resource_quota = cls(
            cpu=cpu,
            memory=memory,
        )

        resource_quota.additional_properties = d
        return resource_quota

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.node_selector_term import NodeSelectorTerm


T = TypeVar("T", bound="NodeSelector")


@_attrs_define
class NodeSelector:
    """
    Attributes:
        node_selector_terms (Union[Unset, list['NodeSelectorTerm']]):
    """

    node_selector_terms: Union[Unset, list["NodeSelectorTerm"]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        node_selector_terms: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.node_selector_terms, Unset):
            node_selector_terms = []
            for node_selector_terms_item_data in self.node_selector_terms:
                node_selector_terms_item = node_selector_terms_item_data.to_dict()
                node_selector_terms.append(node_selector_terms_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if node_selector_terms is not UNSET:
            field_dict["nodeSelectorTerms"] = node_selector_terms

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.node_selector_term import NodeSelectorTerm

        d = dict(src_dict)
        node_selector_terms = []
        _node_selector_terms = d.pop("nodeSelectorTerms", UNSET)
        for node_selector_terms_item_data in _node_selector_terms or []:
            node_selector_terms_item = NodeSelectorTerm.from_dict(
                node_selector_terms_item_data
            )

            node_selector_terms.append(node_selector_terms_item)

        node_selector = cls(
            node_selector_terms=node_selector_terms,
        )

        node_selector.additional_properties = d
        return node_selector

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import Mapping
from typing import (
    Any,
    TypeVar,
    Union,
    cast,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateUserRequest")


@_attrs_define
class UpdateUserRequest:
    """
    Attributes:
        name (Union[Unset, str]):
        avatar_url (Union[Unset, str]):
        default_team_id (Union[None, Unset, str]):
    """

    name: Union[Unset, str] = UNSET
    avatar_url: Union[Unset, str] = UNSET
    default_team_id: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        avatar_url = self.avatar_url

        default_team_id: Union[None, Unset, str]
        if isinstance(self.default_team_id, Unset):
            default_team_id = UNSET
        else:
            default_team_id = self.default_team_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if avatar_url is not UNSET:
            field_dict["avatar_url"] = avatar_url
        if default_team_id is not UNSET:
            field_dict["default_team_id"] = default_team_id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        name = d.pop("name", UNSET)

        avatar_url = d.pop("avatar_url", UNSET)

        def _parse_default_team_id(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        default_team_id = _parse_default_team_id(d.pop("default_team_id", UNSET))

        update_user_request = cls(
            name=name,
            avatar_url=avatar_url,
            default_team_id=default_team_id,
        )

        update_user_request.additional_properties = d
        return update_user_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Contains all the data models used in inputs/outputs"""

from .add_team_member_request import AddTeamMemberRequest
from .add_team_member_request_role import AddTeamMemberRequestRole
from .affinity import Affinity
from .api_key import APIKey
from .auth_provider import AuthProvider
from .capabilities import Capabilities
from .change_password_request import ChangePasswordRequest
from .claim_request import ClaimRequest
from .claim_response import ClaimResponse
from .container_spec import ContainerSpec
from .context_exec_response import ContextExecResponse
from .context_input_request import ContextInputRequest
from .context_resource_usage import ContextResourceUsage
from .context_response import ContextResponse
from .context_response_env_vars import ContextResponseEnvVars
from .context_stats_response import ContextStatsResponse
from .context_web_socket_done import ContextWebSocketDone
from .context_web_socket_done_type import ContextWebSocketDoneType
from .context_web_socket_input import ContextWebSocketInput
from .context_web_socket_input_type import ContextWebSocketInputType
from .context_web_socket_output import ContextWebSocketOutput
from .context_web_socket_output_source import ContextWebSocketOutputSource
from .context_web_socket_output_type import ContextWebSocketOutputType
from .context_web_socket_resize import ContextWebSocketResize
from .context_web_socket_resize_type import ContextWebSocketResizeType
from .context_web_socket_signal import ContextWebSocketSignal
from .context_web_socket_signal_type import ContextWebSocketSignalType
from .create_api_key_request import CreateAPIKeyRequest
from .create_api_key_request_type import CreateAPIKeyRequestType
from .create_api_key_response import CreateAPIKeyResponse
from .create_cmd_context_request import CreateCMDContextRequest
from .create_context_request import CreateContextRequest
from .create_context_request_env_vars import CreateContextRequestEnvVars
from .create_repl_context_request import CreateREPLContextRequest
from .create_sandbox_volume_request import CreateSandboxVolumeRequest
from .create_snapshot_request import CreateSnapshotRequest
from .create_team_request import CreateTeamRequest
from .env_var import EnvVar
from .error import Error
from .error_envelope import ErrorEnvelope
from .exec_candidate import ExecCandidate
from .exposed_port_config import ExposedPortConfig
from .file_content_response import FileContentResponse
from .file_content_response_encoding import FileContentResponseEncoding
from .file_info import FileInfo
from .file_info_type import FileInfoType
from .file_watch_error import FileWatchError
from .file_watch_error_type import FileWatchErrorType
from .file_watch_event import FileWatchEvent
from .file_watch_event_type import FileWatchEventType
from .file_watch_subscribe_request import FileWatchSubscribeRequest
from .file_watch_subscribe_request_action import FileWatchSubscribeRequestAction
from .file_watch_subscribed import FileWatchSubscribed
from .file_watch_subscribed_type import FileWatchSubscribedType
from .file_watch_unsubscribe_request import FileWatchUnsubscribeRequest
from .file_watch_unsubscribe_request_action import FileWatchUnsubscribeRequestAction
from .file_watch_unsubscribed import FileWatchUnsubscribed
from .file_watch_unsubscribed_type import FileWatchUnsubscribedType
from .identity import Identity
from .label_selector import LabelSelector
from .label_selector_match_labels import LabelSelectorMatchLabels
from .label_selector_requirement import LabelSelectorRequirement
from .lifecycle_policy import LifecyclePolicy
from .login_request import LoginRequest
from .login_response import LoginResponse
from .mount_request import MountRequest
from .mount_response import MountResponse
from .mount_status import MountStatus
from .move_file_request import MoveFileRequest
from .network_egress_policy import NetworkEgressPolicy
from .node_affinity import NodeAffinity
from .node_selector import NodeSelector
from .node_selector_requirement import NodeSelectorRequirement
from .node_selector_term import NodeSelectorTerm
from .object_meta import ObjectMeta
from .object_meta_annotations import ObjectMetaAnnotations
from .object_meta_labels import ObjectMetaLabels
from .pause_sandbox_response import PauseSandboxResponse
from .pod_affinity import PodAffinity
from .pod_affinity_term import PodAffinityTerm
from .pod_spec_override import PodSpecOverride
from .pod_spec_override_node_selector import PodSpecOverrideNodeSelector
from .pool_strategy import PoolStrategy
from .port_spec import PortSpec
from .pre_stop_hook import PreStopHook
from .preferred_scheduling_term import PreferredSchedulingTerm
from .process_type import ProcessType
from .pty_size import PTYSize
from .refresh_request import RefreshRequest
from .refresh_response import RefreshResponse
from .register_request import RegisterRequest
from .registry_credentials import RegistryCredentials
from .repl_config import REPLConfig
from .repl_env_var import REPLEnvVar
from .repl_prompt_config import REPLPromptConfig
from .repl_ready_config import REPLReadyConfig
from .repl_ready_mode import REPLReadyMode
from .resize_context_request import ResizeContextRequest
from .resource_quota import ResourceQuota
from .resource_usage import ResourceUsage
from .resume_sandbox_response import ResumeSandboxResponse
from .sandbox import Sandbox
from .sandbox_config import SandboxConfig
from .sandbox_config_env_vars import SandboxConfigEnvVars
from .sandbox_resource_usage import SandboxResourceUsage
from .sandbox_status import SandboxStatus
from .sandbox_template import SandboxTemplate
from .sandbox_template_condition import SandboxTemplateCondition
from .sandbox_template_spec import SandboxTemplateSpec
from .sandbox_template_spec_env_vars import SandboxTemplateSpecEnvVars
from .sandbox_template_status import SandboxTemplateStatus
from .sandbox_update_request import SandboxUpdateRequest
from .sandbox_volume import SandboxVolume
from .security_context import SecurityContext
from .signal_context_request import SignalContextRequest
from .snapshot import Snapshot
from .success_api_key_list_response import SuccessAPIKeyListResponse
from .success_api_key_list_response_data import SuccessAPIKeyListResponseData
from .success_auth_providers_response import SuccessAuthProvidersResponse
from .success_auth_providers_response_data import SuccessAuthProvidersResponseData
from .success_claim_response import SuccessClaimResponse
from .success_context_exec_response import SuccessContextExecResponse
from .success_context_list_response import SuccessContextListResponse
from .success_context_list_response_data import SuccessContextListResponseData
from .success_context_response import SuccessContextResponse
from .success_context_stats_response import SuccessContextStatsResponse
from .success_create_api_key_response import SuccessCreateAPIKeyResponse
from .success_created_response import SuccessCreatedResponse
from .success_created_response_data import SuccessCreatedResponseData
from .success_deleted_response import SuccessDeletedResponse
from .success_deleted_response_data import SuccessDeletedResponseData
from .success_envelope import SuccessEnvelope
from .success_exposed_ports_response import SuccessExposedPortsResponse
from .success_exposed_ports_response_data import SuccessExposedPortsResponseData
from .success_file_list_response import SuccessFileListResponse
from .success_file_list_response_data import SuccessFileListResponseData
from .success_file_read_response import SuccessFileReadResponse
from .success_file_read_response_data_type_1 import SuccessFileReadResponseDataType1
from .success_file_stat_response import SuccessFileStatResponse
from .success_health_response import SuccessHealthResponse
from .success_health_response_data import SuccessHealthResponseData
from .success_identity_list_response import SuccessIdentityListResponse
from .success_identity_list_response_data import SuccessIdentityListResponseData
from .success_login_response import SuccessLoginResponse
from .success_message_response import SuccessMessageResponse
from .success_message_response_data import SuccessMessageResponseData
from .success_mount_response import SuccessMountResponse
from .success_mount_status_response import SuccessMountStatusResponse
from .success_mount_status_response_data import SuccessMountStatusResponseData
from .success_moved_response import SuccessMovedResponse
from .success_moved_response_data import SuccessMovedResponseData
from .success_pause_sandbox_response import SuccessPauseSandboxResponse
from .success_refresh_response import SuccessRefreshResponse
from .success_registry_credentials_response import SuccessRegistryCredentialsResponse
from .success_resized_response import SuccessResizedResponse
from .success_resized_response_data import SuccessResizedResponseData
from .success_restore_response import SuccessRestoreResponse
from .success_restore_response_data import SuccessRestoreResponseData
from .success_resume_sandbox_response import SuccessResumeSandboxResponse
from .success_sandbox_network_policy_response import SuccessSandboxNetworkPolicyResponse
from .success_sandbox_response import SuccessSandboxResponse
from .success_sandbox_status_response import SuccessSandboxStatusResponse
from .success_sandbox_volume_list_response import SuccessSandboxVolumeListResponse
from .success_sandbox_volume_response import SuccessSandboxVolumeResponse
from .success_signaled_response import SuccessSignaledResponse
from .success_signaled_response_data import SuccessSignaledResponseData
from .success_snapshot_list_response import SuccessSnapshotListResponse
from .success_snapshot_response import SuccessSnapshotResponse
from .success_team_list_response import SuccessTeamListResponse
from .success_team_list_response_data import SuccessTeamListResponseData
from .success_team_member_list_response import SuccessTeamMemberListResponse
from .success_team_member_list_response_data import SuccessTeamMemberListResponseData
from .success_team_member_response import SuccessTeamMemberResponse
from .success_team_response import SuccessTeamResponse
from .success_template_list_response import SuccessTemplateListResponse
from .success_template_list_response_data import SuccessTemplateListResponseData
from .success_template_response import SuccessTemplateResponse
from .success_unmounted_response import SuccessUnmountedResponse
from .success_unmounted_response_data import SuccessUnmountedResponseData
from .success_user_response import SuccessUserResponse
from .success_written_response import SuccessWrittenResponse
from .success_written_response_data import SuccessWrittenResponseData
from .team import Team
from .team_member import TeamMember
from .template import Template
from .template_create_request import TemplateCreateRequest
from .template_update_request import TemplateUpdateRequest
from .toleration import Toleration
from .tpl_sandbox_network_policy import TplSandboxNetworkPolicy
from .tpl_sandbox_network_policy_mode import TplSandboxNetworkPolicyMode
from .unmount_request import UnmountRequest
from .update_exposed_ports_request import UpdateExposedPortsRequest
from .update_team_member_request import UpdateTeamMemberRequest
from .update_team_member_request_role import UpdateTeamMemberRequestRole
from .update_team_request import UpdateTeamRequest
from .update_user_request import UpdateUserRequest
from .user import User
from .volume_access_mode import VolumeAccessMode
from .volume_config import VolumeConfig
from .webhook_config import WebhookConfig
from .weighted_pod_affinity_term import WeightedPodAffinityTerm

__all__ = (
    "AddTeamMemberRequest",
    "AddTeamMemberRequestRole",
    "Affinity",
    "APIKey",
    "AuthProvider",
    "Capabilities",
    "ChangePasswordRequest",
    "ClaimRequest",
    "ClaimResponse",
    "ContainerSpec",
    "ContextExecResponse",
    "ContextInputRequest",
    "ContextResourceUsage",
    "ContextResponse",
    "ContextResponseEnvVars",
    "ContextStatsResponse",
    "ContextWebSocketDone",
    "ContextWebSocketDoneType",
    "ContextWebSocketInput",
    "ContextWebSocketInputType",
    "ContextWebSocketOutput",
    "ContextWebSocketOutputSource",
    "ContextWebSocketOutputType",
    "ContextWebSocketResize",
    "ContextWebSocketResizeType",
    "ContextWebSocketSignal",
    "ContextWebSocketSignalType",
    "CreateAPIKeyRequest",
    "CreateAPIKeyRequestType",
    "CreateAPIKeyResponse",
    "CreateCMDContextRequest",
    "CreateContextRequest",
    "CreateContextRequestEnvVars",
    "CreateREPLContextRequest",
    "CreateSandboxVolumeRequest",
    "CreateSnapshotRequest",
    "CreateTeamRequest",
    "EnvVar",
    "Error",
    "ErrorEnvelope",
    "ExecCandidate",
    "ExposedPortConfig",
    "FileContentResponse",
    "FileContentResponseEncoding",
    "FileInfo",
    "FileInfoType",
    "FileWatchError",
    "FileWatchErrorType",
    "FileWatchEvent",
    "FileWatchEventType",
    "FileWatchSubscribed",
    "FileWatchSubscribedType",
    "FileWatchSubscribeRequest",
    "FileWatchSubscribeRequestAction",
    "FileWatchUnsubscribed",
    "FileWatchUnsubscribedType",
    "FileWatchUnsubscribeRequest",
    "FileWatchUnsubscribeRequestAction",
    "Identity",
    "LabelSelector",
    "LabelSelectorMatchLabels",
    "LabelSelectorRequirement",
    "LifecyclePolicy",
    "LoginRequest",
    "LoginResponse",
    "MountRequest",
    "MountResponse",
    "MountStatus",
    "MoveFileRequest",
    "NetworkEgressPolicy",
    "NodeAffinity",
    "NodeSelector",
    "NodeSelectorRequirement",
    "NodeSelectorTerm",
    "ObjectMeta",
    "ObjectMetaAnnotations",
    "ObjectMetaLabels",
    "PauseSandboxResponse",
    "PodAffinity",
    "PodAffinityTerm",
    "PodSpecOverride",
    "PodSpecOverrideNodeSelector",
    "PoolStrategy",
    "PortSpec",
    "PreferredSchedulingTerm",
    "PreStopHook",
    "ProcessType",
    "PTYSize",
    "RefreshRequest",
    "RefreshResponse",
    "RegisterRequest",
    "RegistryCredentials",
    "REPLConfig",
    "REPLEnvVar",
    "REPLPromptConfig",
    "REPLReadyConfig",
    "REPLReadyMode",
    "ResizeContextRequest",
    "ResourceQuota",
    "ResourceUsage",
    "ResumeSandboxResponse",
    "Sandbox",
    "SandboxConfig",
    "SandboxConfigEnvVars",
    "SandboxResourceUsage",
    "SandboxStatus",
    "SandboxTemplate",
    "SandboxTemplateCondition",
    "SandboxTemplateSpec",
    "SandboxTemplateSpecEnvVars",
    "SandboxTemplateStatus",
    "SandboxUpdateRequest",
    "SandboxVolume",
    "SecurityContext",
    "SignalContextRequest",
    "Snapshot",
    "SuccessAPIKeyListResponse",
    "SuccessAPIKeyListResponseData",
    "SuccessAuthProvidersResponse",
    "SuccessAuthProvidersResponseData",
    "SuccessClaimResponse",
    "SuccessContextExecResponse",
    "SuccessContextListResponse",
    "SuccessContextListResponseData",
    "SuccessContextResponse",
    "SuccessContextStatsResponse",
    "SuccessCreateAPIKeyResponse",
    "SuccessCreatedResponse",
    "SuccessCreatedResponseData",
    "SuccessDeletedResponse",
    "SuccessDeletedResponseData",
    "SuccessEnvelope",
    "SuccessExposedPortsResponse",
    "SuccessExposedPortsResponseData",
    "SuccessFileListResponse",
    "SuccessFileListResponseData",
    "SuccessFileReadResponse",
    "SuccessFileReadResponseDataType1",
    "SuccessFileStatResponse",
    "SuccessHealthResponse",
    "SuccessHealthResponseData",
    "SuccessIdentityListResponse",
    "SuccessIdentityListResponseData",
    "SuccessLoginResponse",
    "SuccessMessageResponse",
    "SuccessMessageResponseData",
    "SuccessMountResponse",
    "SuccessMountStatusResponse",
    "SuccessMountStatusResponseData",
    "SuccessMovedResponse",
    "SuccessMovedResponseData",
    "SuccessPauseSandboxResponse",
    "SuccessRefreshResponse",
    "SuccessRegistryCredentialsResponse",
    "SuccessResizedResponse",
    "SuccessResizedResponseData",
    "SuccessRestoreResponse",
    "SuccessRestoreResponseData",
    "SuccessResumeSandboxResponse",
    "SuccessSandboxNetworkPolicyResponse",
    "SuccessSandboxResponse",
    "SuccessSandboxStatusResponse",
    "SuccessSandboxVolumeListResponse",
    "SuccessSandboxVolumeResponse",
    "SuccessSignaledResponse",
    "SuccessSignaledResponseData",
    "SuccessSnapshotListResponse",
    "SuccessSnapshotResponse",
    "SuccessTeamListResponse",
    "SuccessTeamListResponseData",
    "SuccessTeamMemberListResponse",
    "SuccessTeamMemberListResponseData",
    "SuccessTeamMemberResponse",
    "SuccessTeamResponse",
    "SuccessTemplateListResponse",
    "SuccessTemplateListResponseData",
    "SuccessTemplateResponse",
    "SuccessUnmountedResponse",
    "SuccessUnmountedResponseData",
    "SuccessUserResponse",
    "SuccessWrittenResponse",
    "SuccessWrittenResponseData",
    "Team",
    "TeamMember",
    "Template",
    "TemplateCreateRequest",
    "TemplateUpdateRequest",
    "Toleration",
    "TplSandboxNetworkPolicy",
    "TplSandboxNetworkPolicyMode",
    "UnmountRequest",
    "UpdateExposedPortsRequest",
    "UpdateTeamMemberRequest",
    "UpdateTeamMemberRequestRole",
    "UpdateTeamRequest",
    "UpdateUserRequest",
    "User",
    "VolumeAccessMode",
    "VolumeConfig",
    "WebhookConfig",
    "WeightedPodAffinityTerm",
)



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.template import Template


T = TypeVar("T", bound="SuccessTemplateListResponseData")


@_attrs_define
class SuccessTemplateListResponseData:
    """
    Attributes:
        templates (Union[Unset, list['Template']]):
        count (Union[Unset, int]):
    """

    templates: Union[Unset, list["Template"]] = UNSET
    count: Union[Unset, int] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        templates: Union[Unset, list[dict[str, Any]]] = UNSET
        if not isinstance(self.templates, Unset):
            templates = []
            for templates_item_data in self.templates:
                templates_item = templates_item_data.to_dict()
                templates.append(templates_item)

        count = self.count

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if templates is not UNSET:
            field_dict["templates"] = templates
        if count is not UNSET:
            field_dict["count"] = count

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.template import Template

        d = dict(src_dict)
        templates = []
        _templates = d.pop("templates", UNSET)
        for templates_item_data in _templates or []:
            templates_item = Template.from_dict(templates_item_data)

            templates.append(templates_item)

        count = d.pop("count", UNSET)

        success_template_list_response_data = cls(
            templates=templates,
            count=count,
        )

        success_template_list_response_data.additional_properties = d
        return success_template_list_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from dataclasses import dataclass
from typing import Optional


@dataclass(frozen=True)
class RunResult:
    sandbox_id: str
    context_id: str
    output_raw: str


@dataclass(frozen=True)
class CmdResult:
    sandbox_id: str
    context_id: str
    output_raw: str


@dataclass(frozen=True)
class StreamInput:
    type: str = "input"
    data: str = ""
    rows: int = 0
    cols: int = 0
    signal: str = ""
    request_id: str = ""


@dataclass(frozen=True)
class StreamOutput:
    sandbox_id: str
    context_id: str
    source: str
    data: str


@dataclass(frozen=True)
class ExposedPort:
    port: int
    resume: bool
    public_url: str = ""


@dataclass(frozen=True)
class ExposedPortsResponse:
    ports: list[ExposedPort]
    exposure_domain: str = ""


@dataclass(frozen=True)
class FileWatchResponse:
    type: str
    watch_id: str = ""
    event: str = ""
    path: str = ""
    error: str = ""



from collections.abc import Mapping
from typing import (
    TYPE_CHECKING,
    Any,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.exposed_port_config import ExposedPortConfig


T = TypeVar("T", bound="SuccessExposedPortsResponseData")


@_attrs_define
class SuccessExposedPortsResponseData:
    """
    Attributes:
        sandbox_id (str):
        exposed_ports (list['ExposedPortConfig']):
        exposure_domain (Union[Unset, str]): The base exposure domain (e.g., "aws-us-east-1.sandbox0.app").
            Useful for clients that need to construct URLs manually.
    """

    sandbox_id: str
    exposed_ports: list["ExposedPortConfig"]
    exposure_domain: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        sandbox_id = self.sandbox_id

        exposed_ports = []
        for exposed_ports_item_data in self.exposed_ports:
            exposed_ports_item = exposed_ports_item_data.to_dict()
            exposed_ports.append(exposed_ports_item)

        exposure_domain = self.exposure_domain

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sandbox_id": sandbox_id,
                "exposed_ports": exposed_ports,
            }
        )
        if exposure_domain is not UNSET:
            field_dict["exposure_domain"] = exposure_domain

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.exposed_port_config import ExposedPortConfig

        d = dict(src_dict)
        sandbox_id = d.pop("sandbox_id")

        exposed_ports = []
        _exposed_ports = d.pop("exposed_ports")
        for exposed_ports_item_data in _exposed_ports:
            exposed_ports_item = ExposedPortConfig.from_dict(exposed_ports_item_data)

            exposed_ports.append(exposed_ports_item)

        exposure_domain = d.pop("exposure_domain", UNSET)

        success_exposed_ports_response_data = cls(
            sandbox_id=sandbox_id,
            exposed_ports=exposed_ports,
            exposure_domain=exposure_domain,
        )

        success_exposed_ports_response_data.additional_properties = d
        return success_exposed_ports_response_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
